@@ -16,6 +16,7 @@ import (
 	"github.com/spf13/cobra"
 	"gotest.tools/assert"
 
+	"github.com/skupperproject/skupper/api/types"
 	"github.com/skupperproject/skupper/client"
 	"github.com/skupperproject/skupper/pkg/kube"
 	"github.com/skupperproject/skupper/pkg/utils"
@@ -319,7 +320,7 @@ func TestConnectionTokenWithEdgeCluster(t *testing.T) {
 			args:            []string{"/tmp/foo.yaml"},
 			expectedCapture: "",
 			expectedOutput:  "",
-			expectedError:   "Failed to create connection token: Edge configuration cannot accept connections",
+			expectedError:   "Failed to create token: Edge configuration cannot accept connections",
 			realCluster:     true,
 		},
 	}
@@ -374,7 +375,7 @@ func TestConnectionTokenWithInteriorCluster(t *testing.T) {
 		{
 			doc:             "connection-token-test3",
 			args:            []string{"/tmp/foo.yaml"},
-			expectedCapture: "Connection token written to /tmp/foo.yaml",
+			expectedCapture: "Token written to /tmp/foo.yaml",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
@@ -423,7 +424,7 @@ func TestConnectWithEdgeCluster(t *testing.T) {
 		{
 			doc:             "connect-test2",
 			args:            []string{"/tmp/foo.yaml", "--connection-name", "edge-conn1"},
-			expectedCapture: "Skupper configured to connect to",
+			expectedCapture: "Site configured to link to",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
@@ -471,7 +472,7 @@ func TestConnectWithInteriorCluster(t *testing.T) {
 		{
 			doc:             "connect-test2",
 			args:            []string{"/tmp/foo.yaml", "--connection-name", "interior-conn1"},
-			expectedCapture: "Skupper configured to connect to",
+			expectedCapture: "Site configured to link to",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
@@ -518,11 +519,20 @@ func TestDisconnectWithCluster(t *testing.T) {
 		},
 		{
 			doc:             "disconnect-test2",
-			args:            []string{"conn1"},
-			expectedCapture: "Link 'conn1' has been removed",
+			args:            []string{"link1"},
+			expectedCapture: "",
+			expectedOutput:  "",
+			expectedError:   "No such link \"link1\"",
+			realCluster:     true,
+		},
+		{
+			doc:             "disconnect-test3",
+			args:            []string{"link1"},
+			expectedCapture: "Link 'link1' has been removed",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
+			createConn:      true,
 		},
 	}
 
@@ -547,7 +557,20 @@ func TestDisconnectWithCluster(t *testing.T) {
 		if tc.realCluster && !*clusterRun {
 			continue
 		}
-
+		if tc.createConn {
+			if c, ok := cli.(*client.VanClient); ok {
+				token := &corev1.Secret{
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "link1",
+						Labels: map[string]string{
+							types.SkupperTypeQualifier: types.TypeToken,
+						},
+					},
+				}
+				_, err := c.KubeClient.CoreV1().Secrets(namespace).Create(token)
+				assert.Check(t, err)
+			}
+		}
 		cmd := NewCmdDisconnect(testClient)
 		silenceCobra(cmd)
 		testCommand(t, cmd, tc.doc, tc.expectedError, tc.expectedCapture, tc.expectedOutput, tc.args...)
@@ -611,7 +634,7 @@ func TestListConnectorsWithCluster(t *testing.T) {
 		if tc.createConn {
 			connectCmd := NewCmdConnect(testClient)
 			silenceCobra(connectCmd)
-			testCommand(t, connectCmd, tc.doc, "", "Skupper configured to connect to", "", []string{"/tmp/foo.yaml"}...)
+			testCommand(t, connectCmd, tc.doc, "", "Site configured to link to", "", []string{"/tmp/foo.yaml"}...)
 		}
 
 		cmd := NewCmdListConnectors(testClient)
@@ -633,23 +656,23 @@ func TestCheckConnectionWithCluster(t *testing.T) {
 		{
 			doc:             "check-connection-test2",
 			args:            []string{"all"},
-			expectedCapture: "There are no connectors configured or active",
+			expectedCapture: "There are no links configured or active",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
 		},
 		{
 			doc:             "check-connection-test3",
-			args:            []string{"conn1"},
-			expectedCapture: "There are no connectors configured or active",
+			args:            []string{"link1"},
+			expectedCapture: "No such link",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
 		},
 		{
 			doc:             "check-connection-test4",
-			args:            []string{"conn1"},
-			expectedCapture: "Connection for conn1 not active",
+			args:            []string{"link1"},
+			expectedCapture: "Link link1 not active",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
@@ -658,7 +681,7 @@ func TestCheckConnectionWithCluster(t *testing.T) {
 		{
 			doc:             "check-connection-test5",
 			args:            []string{"all"},
-			expectedCapture: "Connection for conn1 not active",
+			expectedCapture: "Link link1 not active",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
@@ -690,7 +713,7 @@ func TestCheckConnectionWithCluster(t *testing.T) {
 		if tc.createConn {
 			cmd := NewCmdConnect(testClient)
 			silenceCobra(cmd)
-			testCommand(t, cmd, tc.doc, "", "Skupper configured to connect to", "", []string{"/tmp/foo.yaml"}...)
+			testCommand(t, cmd, tc.doc, "", "Site configured to link to", "", []string{"/tmp/foo.yaml"}...)
 		}
 
 		cmd := NewCmdCheckConnection(testClient)
@@ -1028,7 +1051,7 @@ func TestListExposedWithCluster(t *testing.T) {
 			doc:             "list-exposed-test1",
 			args:            []string{"--help"},
 			expectedCapture: "",
-			expectedOutput:  "List services exposed over the Skupper network",
+			expectedOutput:  "List services exposed over the service network",
 			expectedError:   "",
 			realCluster:     false,
 		},
@@ -1411,14 +1434,22 @@ func TestDebugDumpWithCluster(t *testing.T) {
 			doc:             "debug-dump-test1",
 			args:            []string{"--help"},
 			expectedCapture: "",
-			expectedOutput:  "Collect and save skupper logs, config, etc.",
+			expectedOutput:  "Collect and store skupper logs, config, etc. to compressed archive file",
 			expectedError:   "",
 			realCluster:     false,
 		},
 		{
 			doc:             "debug-dump-test2",
 			args:            []string{"./tmp/dump.txt"},
-			expectedCapture: "",
+			expectedCapture: "Skupper dump details written to compressed archive:  ./tmp/dump.txt",
+			expectedOutput:  "",
+			expectedError:   "",
+			realCluster:     true,
+		},
+		{
+			doc:             "debug-dump-test3",
+			args:            []string{"./tmp/dump"},
+			expectedCapture: "Skupper dump details written to compressed archive:  ./tmp/dump.tar.gz",
 			expectedOutput:  "",
 			expectedError:   "",
 			realCluster:     true,
