@@ -8,6 +8,7 @@ import (
 	"testing"
 
 	"github.com/skupperproject/skupper/test/utils/base"
+	"gotest.tools/assert"
 	_ "k8s.io/client-go/plugin/pkg/client/auth"
 )
 
@@ -23,9 +24,13 @@ func TestBookinfo(t *testing.T) {
 		PrivateClusters: 1,
 	}
 	testRunner := &base.ClusterTestRunnerBase{}
-	testRunner.BuildOrSkip(t, needs, nil)
+	if err := testRunner.Validate(needs); err != nil {
+		t.Skipf("%s", err)
+	}
+	_, err := testRunner.Build(needs, nil)
+	assert.Assert(t, err)
 	ctx, cancel := context.WithCancel(context.Background())
-	base.HandleInterruptSignal(t, func(t *testing.T) {
+	base.HandleInterruptSignal(func() {
 		base.TearDownSimplePublicAndPrivate(testRunner)
 		cancel()
 	})
