@@ -8,6 +8,7 @@ import (
 	"testing"
 
 	"github.com/skupperproject/skupper/test/utils/base"
+	"gotest.tools/assert"
 )
 
 func TestMain(m *testing.M) {
@@ -22,9 +23,13 @@ func TestMongo(t *testing.T) {
 		PrivateClusters: 1,
 	}
 	testRunner := &base.ClusterTestRunnerBase{}
-	testRunner.BuildOrSkip(t, needs, nil)
+	if err := testRunner.Validate(needs); err != nil {
+		t.Skipf("%s", err)
+	}
+	_, err := testRunner.Build(needs, nil)
+	assert.Assert(t, err)
 	//ctx, cancel := context.WithCancel(context.Background())
-	//base.HandleInterruptSignal(t, func(t *testing.T) {
+	//base.HandleInterruptSignal(func() {
 	//base.TearDownSimplePublicAndPrivate(&testRunner.ClusterTestRunnerBase)
 	//cancel()
 	//})
